/* -*- c++ -*- */
/*
 * Copyright 2014 <+YOU OR YOUR COMPANY+>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnuradio/io_signature.h>
#include "c_fir_filter_impl.h"
#include <volk/volk.h>

namespace gr {
  namespace mymod {

    c_fir_filter::sptr
    c_fir_filter::make(int decim, const std::vector<float> &taps)
    {
      return gnuradio::get_initial_sptr
        (new c_fir_filter_impl(decim, taps));
    }

    /*
     * The private constructor
     */
    c_fir_filter_impl::c_fir_filter_impl(int decim, const std::vector<float> &taps)
      : gr::sync_decimator("c_fir_filter",
                           gr::io_signature::make(1, 1, sizeof(float)),
                           gr::io_signature::make(1, 1, sizeof(float)), decim)
    {
      d_taps = taps;
      std::reverse(d_taps.begin(), d_taps.end());

      const int alignment_multiple =
	volk_get_alignment() / sizeof(float);
      set_alignment(std::max(1, alignment_multiple));
    }

    /*
     * Our virtual destructor.
     */
    c_fir_filter_impl::~c_fir_filter_impl()
    {
    }

    int
    c_fir_filter_impl::work(int noutput_items,
			  gr_vector_const_void_star &input_items,
			  gr_vector_void_star &output_items)
    {
        const float *in = (const float *)input_items[0];
        float *out = (float *)output_items[0];

        // Warning. Beware bugs in the following code; I have only
        // been proved it correct, not tried it.
        //    -Knuth

        // Non-decimating version
        int n = 0;
        for(int i = 0; i < noutput_items; i++) {
          out[i] = 0;
          for(size_t j = 0; j < d_taps.size(); j++) {
            out[i] += in[n+j]*d_taps[j];
          }
          n++;
        }






        // decimating version
        int n = decimation()-1;
        for(int i = 0; i < noutput_items; i++) {
          out[i] = 0;
          for(size_t j = 0; j < d_taps.size(); j++) {
            out[i] += in[n+j]*d_taps[j];
          }
          n+=decimation();
        }



        // Decimating filter with VOLK
        int n = 0;
        for(int i = 0; i < noutput_items; i++) {
          volk_32f_x2_dot_prod_32f(&out[i], &in[n], &d_taps[0],
                                 d_taps.size());
          n+=decimation();
        }
        return noutput_items;
    }

  } /* namespace mymod */
} /* namespace gr */
