/* -*- c++ -*- */
/*
 * Copyright 2014 <+YOU OR YOUR COMPANY+>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnuradio/io_signature.h>
#include "energy_detect_impl.h"

namespace gr {
  namespace mymod {

    energy_detect::sptr
    energy_detect::make()
    {
      return gnuradio::get_initial_sptr
        (new energy_detect_impl());
    }

    /*
     * The private constructor
     */
    energy_detect_impl::energy_detect_impl()
      : gr::block("energy_detect",
                  gr::io_signature::make(1, 1, sizeof(gr_complex)),
                  gr::io_signature::make(0, 0, 0))
    {
      d_thresh = 0;
      d_state = false;
      d_cal = true;
      message_port_register_out(pmt::intern("det"));
      message_port_register_in(pmt::intern("cal"));
      set_msg_handler(pmt::intern("cal"),
                      boost::bind(&energy_detect_impl::calibrate, this, _1));
      d_iir.set_taps(0.001);
    }

    energy_detect_impl::~energy_detect_impl()
    {
    }

    void
    energy_detect_impl::calibrate(pmt::pmt_t msg)
    {
      if(pmt::is_bool(msg)) {
        d_cal = pmt::to_bool(msg);
      }
    }

    int
    energy_detect_impl::general_work(int noutput_items,
                                     gr_vector_int &ninput_items,
                                     gr_vector_const_void_star &input_items,
                                     gr_vector_void_star &output_items)
    {
      const gr_complex *in = (const gr_complex*)input_items[0];

      if(d_cal) {
        for(int i = 0; i < noutput_items; i++) {
          d_thresh = d_iir.filter(abs(in[i]));
        }
      }
      else {
        double v;
        for(int i = 0; i < noutput_items; i++) {
          v = d_iir.filter(abs(in[i]));

          if(!d_state) {
            if(v > d_thresh + 0.05) {
              std::cerr << "ABOVE THRESHOLD (" << d_thresh << "): " << v << std::endl;
              d_state = true;
              message_port_pub(pmt::intern("det"), pmt::from_bool(true));
            }
          }
          else {
            if(v < d_thresh) {
              std::cerr << "BELOW THRESHOLD" << std::endl;
              d_state = false;
              message_port_pub(pmt::intern("det"), pmt::from_bool(false));
            }
          }
        }
      }

      consume_each(noutput_items);
      return noutput_items;
    }

  } /* namespace mymod */
} /* namespace gr */
